@@ -33,7 +33,13 @@ pub enum Error {
     /// The given identifier had more than one `:`, rendering it invalid.
     InvalidIdentifier,
     /// A given ID for an Enum was out of valid bounds for that type.
-    EnumOutOfBound
+    EnumOutOfBound,
+    /// An error occured parsing JSON data using `serde_json`.
+    JsonParsingError(serde_json::Error),
+    /// A JSON tag had a weird root structure.
+    InvalidJsonRoot,
+    /// A UUID consited of characters other than 0-f
+    InvalidUUID(std::num::ParseIntError)
 }
 
 impl std::fmt::Display for Error {
@@ -42,8 +48,235 @@ impl std::fmt::Display for Error {
     }
 }
 
+impl From<serde_json::Error> for Error {
+    fn from(e: serde_json::Error) -> Error {
+        return Error::JsonParsingError(e);
+    }
+}
+
+impl From<std::num::ParseIntError> for Error {
+    fn from(e: std::num::ParseIntError) -> Error {
+        return Error::InvalidUUID(e);
+    }
+}
+
 impl std::error::Error for Error {}
 
+/// Represents a Unique User ID. Used to track players and entities.
+#[derive(Debug, Clone, Eq, PartialEq)]
+pub struct UUID {
+    /// The value of this UUID
+    value: u128
+}
+
+impl UUID {
+    /// Generates a UUID from a Read type.
+    pub fn from_reader<R: std::io::Read>(reader: &mut R) -> Result<UUID, Error> {
+        return Ok(Self::from_bytes(&[read_byte(reader)?; 16])?.0);
+    }
+    /// Generates a UUID from a byte array. Returns the UUID and amount of bytes needed.
+    pub fn from_bytes(data: &[u8]) -> Result<(UUID, usize), Error> {
+        if data.len() < 16 {
+            return Err(Error::MissingData);
+        }
+        let mut array = [0;16];
+        for i in 0..16 {
+            array[i] = data[i];
+        }
+        return Ok((Self::from_value(u128::from_be_bytes(array))?, 16));
+    }
+    /// Generates a UUID from a given value.
+    pub fn from_value(value: u128) -> Result<UUID, Error> {
+        return Ok(UUID { value });
+    }
+    /// Generates a UUID from a username. This function uses Mojang's API, and may be subject to
+    /// rate limiting. Cache your results.
+    pub fn from_username(username: String) -> Result<UUID, Error> {
+        use reqwest::blocking::get;
+        let raw_response = get(format!("https://api.mojang.com/users/profiles/minecraft/{}", username)).unwrap().text().unwrap();
+        let json_response: serde_json::Value = serde_json::from_str(&raw_response)?;
+        return Self::from_value(u128::from_str_radix(&json_response["id"].as_str().ok_or(Error::InvalidJsonRoot)?, 16)?);
+    }
+    /// Writes this UUID to a Write type.
+    pub fn to_writer<W: std::io::Write>(self, writer: &mut W) -> Result<(), Error> {
+        match writer.write_all(&self.value.to_be_bytes()) {
+            Ok(_) => {},
+            Err(e) => {
+                return Err(Error::WriterError(e));
+            }
+        }
+        return Ok(());
+    }
+    /// Creates a byte array with the data of this UUID in it.
+    pub fn to_bytes(self) -> Result<Vec<u8>, Error> {
+        return Ok(self.value.to_be_bytes().to_vec());
+    }
+    /// Gives the underlying value of this UUID.
+    pub fn to_value(self) -> Result<u128, Error> {
+        return Ok(self.value);
+    }
+    /// Gives the username associated with this UUID. This function uses Mojang's API, and may be
+    /// subject to rate limiting. Cache your results.
+    pub fn to_username(self) -> Result<String, Error> {
+        use reqwest::blocking::get;
+        let mut insertable = format!("{:x}", self.value);
+        insertable = insertable.split('x').next_back().unwrap().to_string();
+        while insertable.len() < 32 {
+            insertable = String::from("0") + &insertable;
+        }
+        let raw_response = get(format!("https://api.mojang.com/user/profiles/{}/names", insertable)).unwrap().text().unwrap();
+        let json_response: serde_json::Value = serde_json::from_str(&raw_response)?;
+        if json_response.is_array() {
+            let json_response = json_response.as_array().unwrap();
+            return Ok(json_response[json_response.len() - 1]["name"].as_str().ok_or(Error::InvalidJsonRoot)?.to_string());
+        }
+        else {
+            return Err(Error::InvalidJsonRoot);
+        }
+    }
+}
+
+use serde::{Serialize, Deserialize};
+
+#[derive(Debug, Clone, Eq, PartialEq)]
+/// Represents a chat message or other form of rich text.
+pub struct Chat {
+    component: ChatComponent
+}
+
+#[derive(Debug, Clone, Eq, PartialEq, Deserialize, Serialize)]
+#[allow(non_snake_case)]
+/// Represents one component of a Chat object.
+pub struct ChatComponent {
+    /// Text to be used.
+    pub text: Option<String>,
+    /// Translation key to be used.
+    pub translate: Option<String>,
+    /// Key to use the translated keybind for.
+    pub keybind: Option<String>,
+    /// Scoreboard to use.
+    pub score: Option<ChatScore>,
+    /// Selector to use with `score`.
+    pub selector: Option<String>,
+    /// Declares if the text is bold.
+    pub bold: Option<bool>,
+    /// Declares if the text is italic.
+    pub italic: Option<bool>,
+    /// Declares if the text is underlined.
+    pub underlined: Option<bool>,
+    /// Declares if the text has a strikethrough applied to it.
+    pub strikethrough: Option<bool>,
+    /// Declares if the text is obfuscated.
+    pub obfuscated: Option<bool>,
+    /// Declares the color of the text.
+    pub color: Option<String>,
+    /// Declares text to insert into the client's chat when clicked.
+    pub insertion: Option<String>,
+    /// Defines an event when this text is clicked.
+    pub clickEvent: Option<ClickEvent>,
+    /// Defines an event when a client is hovering over this text.
+    pub hoverEvent: Option<HoverEvent>,
+    /// Declares extra components to add aftr this one.
+    pub extra: Option<Vec<ChatComponent>>
+}
+
+#[derive(Debug, Clone, Eq, PartialEq, Deserialize, Serialize)]
+/// Describes details about a scoreboard.
+pub struct ChatScore {
+    /// Name of the given scoreboard.
+    pub name: String,
+    /// Objective of the given scoreboard.
+    pub objective: String,
+    /// Value to assign to the given scoreboard.
+    pub value: Option<String>
+}
+
+#[derive(Debug, Clone, Eq, PartialEq, Deserialize, Serialize)]
+pub struct ClickEvent {
+    pub action: String,
+    pub value: String
+}
+
+#[derive(Debug, Clone, Eq, PartialEq, Deserialize, Serialize)]
+pub struct HoverEvent {
+    pub action: String,
+    pub value: String
+}
+
+impl Chat {
+    pub fn from_bytes(data: &[u8]) -> Result<(Chat, usize), Error> {
+        let string_data = generalized::string_from_bytes(data)?;
+        return Ok((Self::from_string(string_data.0)?, string_data.1));
+    }
+    pub fn from_reader<R: std::io::Read>(read: &mut R) -> Result<Chat, Error> {
+        return Self::from_string(generalized::string_from_reader(read)?);
+    }
+    pub fn from_string(data: String) -> Result<Chat, Error> {
+        let structure: serde_json::Value = serde_json::from_str(&data)?;
+        if structure.is_object() {
+            return Ok(Chat {
+                component: serde_json::from_str(&data)?
+            });
+        }
+        else if structure.is_array() {
+            return Ok(Chat {
+                component: ChatComponent {
+                    text: None,
+                    translate: None,
+                    keybind: None,
+                    score: None,
+                    selector: None,
+                    bold: None,
+                    italic: None,
+                    underlined: None,
+                    strikethrough: None,
+                    obfuscated: None,
+                    color: None,
+                    insertion: None,
+                    clickEvent: None,
+                    hoverEvent: None,
+                    extra: serde_json::from_str(&data)?
+                }
+            });
+        }
+        else if structure.is_string() {
+            return Ok(Chat {
+                component: ChatComponent {
+                    text: serde_json::from_str(&data)?,
+                    translate: None,
+                    keybind: None,
+                    score: None,
+                    selector: None,
+                    bold: None,
+                    italic: None,
+                    underlined: None,
+                    strikethrough: None,
+                    obfuscated: None,
+                    color: None,
+                    insertion: None,
+                    clickEvent: None,
+                    hoverEvent: None,
+                    extra: None
+                }
+            });
+        }
+        else {
+            return Err(Error::InvalidJsonRoot);
+        }
+    }
+    pub fn to_bytes(self) -> Result<Vec<u8>, Error> {
+        return generalized::string_to_bytes(serde_json::to_string(&self.component)?);
+    }
+    pub fn to_writer<W: std::io::Write>(self, writer: &mut W) -> Result<(), Error> {
+        generalized::string_to_writer(writer, serde_json::to_string(&self.component)?)?;
+        return Ok(());
+    }
+    pub fn to_string(self) -> Result<String, Error> {
+        return Ok(serde_json::to_string(&self.component)?);
+    }
+}
+
+
 /// Provides tools for reading, writing, and managing the various enums that Minecraft uses.
 /// Many of these enums contain descriptions of their respective attributes in quotes. This
 /// indicates that the information is taken directly from https://wiki.vg/Protocol_FAQ
@@ -136,14 +369,37 @@ pub mod enums {
             }
         }
     }
+    #[derive(Clone, Copy, PartialEq, Eq, Debug)]
+    #[repr(i32)]
+    /// Indicates what state to switch to to choose the right section of the protocol.
+    pub enum NextState {
+        /// Switch to the Status state. (used for the server list)
+        Status = 1,
+        /// Switch to the Login state.
+        Login = 2
+    }
+    impl TryFrom<crate::VarInt> for NextState {
+        type Error = Error;
+        fn try_from(value: crate::VarInt) -> Result<Self, Self::Error> {
+            match value {
+                x if x == crate::VarInt::from_value(NextState::Login as i32)? => Ok(NextState::Login),
+                x if x == crate::VarInt::from_value(NextState::Status as i32)? => Ok(NextState::Status),
+                _ => Err(Error::EnumOutOfBound)
+            }
+        }
+    }
 
     #[derive(Clone, Copy, PartialEq, Eq, Debug)]
     #[repr(u8)]
     /// Indicates the current section of the network protocol to use.
     pub enum ProtocolState {
+        /// The Handshake state is used to confirm connection and choose the next state.
         Handshake = 0,
+        /// The Status state is for getting information for use in the server list.
         Status = 1,
+        /// The Login state is for encrypting, compressing, and preparing for the Play state.
         Login = 2,
+        /// The Play state is for standard gameplay.
         Play = 3
     }
     impl TryFrom<u8> for ProtocolState {
@@ -1828,4 +2084,12 @@ mod test {
         assert_eq!(min_value.to_bytes()?, [0x00, 0x00, 0x06, 0x00, 0x00, 0x01, 0x80, 0x0E]);
         return Ok(());
     }
+    #[test]
+    fn username_api() -> Result<(), super::Error> {
+        use super::UUID;
+        let uuid = UUID::from_username(String::from("thisjaiden"))?;
+        assert_eq!(uuid.clone().to_value()?, 0x09773765901b4da1a1243467f482b8b3);
+        assert_eq!(uuid.to_username()?, String::from("thisjaiden"));
+        return Ok(());
+    }
 }
